<?php

/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

define ("SERVICE_NAME", "PHP_BS");

// PHP classes corresponding to the data types in defined in WSDL
/**
 * This class encapsulate information about an order object.
 */
class OrderDataBean 
{
    public $orderID; // int
    public $orderType; // string
    public $orderStatus; // string
    public $openDate; // dateTime
    public $completionDate; // dateTime
    public $quantity; // double
    public $price; // decimal
    public $orderFee; // decimal
    public $symbol; // string
}

/**
 * This class encapsulate an order object to be submitted to order
 * processor.
 */
class SubmitOrder 
{
    public $order; // (object)OrderDataBean
}

/*
 * request send to get information needed by business service
 */
class BSConfigRequest 
{
    public $BS;
}

/*
 * configuration parameters needed by business service
 */
class BSConfigResponse 
{
    public $DBName;
    public $DBHostName;
    public $DBPort;
    public $OPS;
    public $Sec;
}

/**
 * This method will get configurations needed to make connection to order processor service
 * returns BSConfigResponse object
 */
function GetOPSConfig()
{
	// define the class map
	$class_map = array(
		"BSConfigRequest" => "BSConfigRequest",
		"BSConfigResponse" => "BSConfigResponse");

    // create client in WSDL mode
    $client = new WSClient(array ("wsdl" =>"../resources/wsdl/config_svc.wsdl",
		"classmap" => $class_map, 
		"to" => GetConfigServiceEndpoint()));

    // get proxy object reference form client 
    $proxy = $client->getProxy();
	$input = new BSConfigRequest();
	$input->BS = SERVICE_NAME;

    $response = $proxy->BSConfigRequest($input);
	return $response;
}

/**
 * This method returns a proxy object so that all the other operations 
 * can use it for calling Web Service operations.
 * @return a proxy object.
 */
function GetProxy()
{
	$OPSConfig = GetOPSConfig();
	if($OPSConfig)
	{
		$class_map = array(
				"SubmitOrderTransactedQueue" => "SubmitOrderTransactedQueue",
				"OrderDataBean" => "OrderDataBean",
				"SubmitOrder" => "SubmitOrder");
				
		if ($OPSConfig->Sec)
		{
			//This is the security information
			$rec_cert = ws_get_key_from_file("../resources/keys/OPS.cert");
			$policy_xml = file_get_contents("../resources/conf/policy.xml"); 
			$policy = new WSPolicy($policy_xml);
			$sec_token = new WSSecurityToken(array("receiverCertificate" => $rec_cert));
			$client = new WSClient(array (
						"wsdl" => "../resources/wsdl/TradeOrders.wsdl",
						"classmap" => $class_map,
						"policy" => $policy,
						"securityToken" => $sec_token,
						"useSOAP" => "1.2",
						"to" => $OPSConfig->OPS));
		}
		else
		{
			// create client in WSDL mode
			$client = new WSClient(array (
						"wsdl" => "../resources/wsdl/TradeOrders.wsdl",
						"classmap" => $class_map,
						"to" => $OPSConfig->OPS));
		}
		
		// get proxy object reference form client 
		$proxy = $client->getProxy();
	}
	return $proxy;
}

/**
 * This method obtains the config service endpoint from a 
 * configuration file. If it can not retrieve from that, it used the
 * default value.
 * @return the endpoint.
 */
function GetConfigServiceEndpoint()
{
	$doc = new DOMDocument();
	$result = $doc->load('../resources/conf/configuration_service_config.xml');
	$service = 
		$doc->getElementsByTagName("config_service")->item(0)->nodeValue;
	
	if (!($service))
	{
		$service = 
			"http://localhost:8080/php_stocktrader/config_service/config_svc.php";
	}
	return $service;
}

/**
 * This method does the neccessary processing prior to submit the order
 * for further processing.
 * @param order, the order object to be submitted.
 * @return the order object just submitted.
 */
function processSubmitOrder($order)
{	
	$proxy = GetProxy();
	$submitOrder = new SubmitOrder();
	$submitOrder->order = new OrderDataBean();
	$submitOrder->order->orderID = $order->orderID;
	$submitOrder->order->orderType = $order->orderType;
	$submitOrder->order->orderStatus = "open";
	$submitOrder->order->quantity = $order->quantity;
	$submitOrder->order->price = $order->price;
	$submitOrder->order->orderFee = $order->orderFee;
	$submitOrder->order->symbol = $order->symbol;
	$submitOrder->order->openDate = date(DATE_ATOM, time());
	$submitOrder->order->completionDate = "1970-01-01T05:30:00.000+05:30";
	
	if ($proxy)
	{
		$proxy->SubmitOrder($submitOrder);
	}
}
?>
